@@ -1,5 +1,3 @@
-from __future__ import print_function, division, absolute_import
-
 from datetime import timedelta
 import logging
 from multiprocessing.queues import Empty
@@ -17,7 +15,7 @@ from tornado import gen
 from tornado.ioloop import IOLoop, TimeoutError
 from tornado.locks import Event
 
-from .comm import get_address_host, get_local_address_for, unparse_host_port
+from .comm import get_address_host, unparse_host_port
 from .comm.addressing import address_from_user_args
 from .core import RPCClosed, CommClosedError, coerce_to_address
 from .metrics import time
@@ -42,7 +40,15 @@ class Nanny(ServerNode):
     """ A process to manage worker processes
 
     The nanny spins up Worker processes, watches then, and kills or restarts
-    them as necessary.
+    them as necessary. It is necessary if you want to use the
+    ``Client.restart`` method, or to restart the worker automatically if
+    it gets to the terminate fractiom of its memory limit.
+
+    The parameters for the Nanny are mostly the same as those for the Worker.
+
+    See Also
+    --------
+    Worker
     """
 
     _instances = weakref.WeakSet()
@@ -58,7 +64,8 @@ class Nanny(ServerNode):
         nthreads=None,
         ncores=None,
         loop=None,
-        local_dir="dask-worker-space",
+        local_dir=None,
+        local_directory="dask-worker-space",
         services=None,
         name=None,
         memory_limit="auto",
@@ -112,6 +119,14 @@ class Nanny(ServerNode):
         self.preload_argv = preload_argv
         self.Worker = Worker if worker_class is None else worker_class
         self.env = env or {}
+        worker_kwargs.update(
+            {
+                "port": worker_port,
+                "interface": interface,
+                "protocol": protocol,
+                "host": host,
+            }
+        )
         self.worker_kwargs = worker_kwargs
 
         self.contact_address = contact_address
@@ -119,7 +134,11 @@ class Nanny(ServerNode):
             "distributed.worker.memory.terminate"
         )
 
-        self.local_dir = local_dir
+        if local_dir is not None:
+            warnings.warn("The local_dir keyword has moved to local_directory")
+            local_directory = local_dir
+
+        self.local_directory = local_directory
 
         self.services = services
         self.name = name
@@ -153,6 +172,13 @@ class Nanny(ServerNode):
             pc = PeriodicCallback(self.memory_monitor, 100, io_loop=self.loop)
             self.periodic_callbacks["memory"] = pc
 
+        if (
+            not host
+            and not interface
+            and not self.scheduler_addr.startswith("inproc://")
+        ):
+            host = get_ip(get_address_host(self.scheduler.address))
+
         self._start_address = address_from_user_args(
             host=host,
             port=port,
@@ -168,8 +194,7 @@ class Nanny(ServerNode):
     def __repr__(self):
         return "<Nanny: %s, threads: %d>" % (self.worker_address, self.nthreads)
 
-    @gen.coroutine
-    def _unregister(self, timeout=10):
+    async def _unregister(self, timeout=10):
         if self.process is None:
             return
         worker_address = self.process.worker_address
@@ -183,7 +208,7 @@ class Nanny(ServerNode):
             RPCClosed,
         )
         try:
-            yield gen.with_timeout(
+            await gen.with_timeout(
                 timedelta(seconds=timeout),
                 self.scheduler.unregister(address=self.worker_address),
                 quiet_exceptions=allowed_errors,
@@ -199,47 +224,30 @@ class Nanny(ServerNode):
     def worker_dir(self):
         return None if self.process is None else self.process.worker_dir
 
-    @gen.coroutine
-    def _start(self, addr_or_port=0):
-        """ Start nanny, start local process, start watching """
-        addr_or_port = addr_or_port or self._start_address
+    @property
+    def local_dir(self):
+        """ For API compatibility with Nanny """
+        warnings.warn("The local_dir attribute has moved to local_directory")
+        return self.local_directory
 
-        # XXX Factor this out
-        if not addr_or_port:
-            # Default address is the required one to reach the scheduler
-            self.listen(
-                get_local_address_for(self.scheduler.address),
-                listen_args=self.listen_args,
-            )
-            self.ip = get_address_host(self.address)
-        elif isinstance(addr_or_port, int):
-            # addr_or_port is an integer => assume TCP
-            self.ip = get_ip(get_address_host(self.scheduler.address))
-            self.listen((self.ip, addr_or_port), listen_args=self.listen_args)
-        else:
-            self.listen(addr_or_port, listen_args=self.listen_args)
-            self.ip = get_address_host(self.address)
+    async def start(self):
+        """ Start nanny, start local process, start watching """
+        self.listen(self._start_address, listen_args=self.listen_args)
+        self.ip = get_address_host(self.address)
 
         logger.info("        Start Nanny at: %r", self.address)
-        response = yield self.instantiate()
+        response = await self.instantiate()
         if response == "running":
             assert self.worker_address
             self.status = "running"
         else:
-            yield self.close()
+            await self.close()
 
         self.start_periodic_callbacks()
 
-        raise gen.Return(self)
+        return self
 
-    def __await__(self):
-        return self._start().__await__()
-
-    def start(self, addr_or_port=0):
-        self.loop.add_callback(self._start, addr_or_port)
-
-    @gen.coroutine
-    def kill(self, comm=None, timeout=2):
+    async def kill(self, comm=None, timeout=2):
         """ Kill the local worker process
 
         Blocks until both the process is down and the scheduler is properly
@@ -247,13 +255,12 @@ class Nanny(ServerNode):
         """
         self.auto_restart = False
         if self.process is None:
-            raise gen.Return("OK")
+            return "OK"
 
         deadline = self.loop.time() + timeout
-        yield self.process.kill(timeout=0.8 * (deadline - self.loop.time()))
+        await self.process.kill(timeout=0.8 * (deadline - self.loop.time()))
 
-    @gen.coroutine
-    def instantiate(self, comm=None):
+    async def instantiate(self, comm=None):
         """ Start a local worker process
 
         Blocks until the process is up and the scheduler is properly informed
@@ -270,7 +277,7 @@ class Nanny(ServerNode):
             worker_kwargs = dict(
                 scheduler_ip=self.scheduler_addr,
                 nthreads=self.nthreads,
-                local_dir=self.local_dir,
+                local_directory=self.local_directory,
                 services=self.services,
                 nanny=self.address,
                 name=self.name,
@@ -287,11 +294,10 @@ class Nanny(ServerNode):
             )
             worker_kwargs.update(self.worker_kwargs)
             self.process = WorkerProcess(
-                worker_args=tuple(),
                 worker_kwargs=worker_kwargs,
                 worker_start_args=(start_arg,),
                 silence_logs=self.silence_logs,
-                on_exit=self._on_exit,
+                on_exit=self._on_exit_sync,
                 worker=self.Worker,
                 env=self.env,
             )
@@ -299,11 +305,11 @@ class Nanny(ServerNode):
         self.auto_restart = True
         if self.death_timeout:
             try:
-                result = yield gen.with_timeout(
+                result = await gen.with_timeout(
                     timedelta(seconds=self.death_timeout), self.process.start()
                 )
             except gen.TimeoutError:
-                yield self.close(timeout=self.death_timeout)
+                await self.close(timeout=self.death_timeout)
                 logger.exception(
                     "Timed out connecting Nanny '%s' to scheduler '%s'",
                     self,
@@ -312,26 +318,24 @@ class Nanny(ServerNode):
                 raise
 
         else:
-            result = yield self.process.start()
-        raise gen.Return(result)
+            result = await self.process.start()
+        return result
 
-    @gen.coroutine
-    def restart(self, comm=None, timeout=2, executor_wait=True):
+    async def restart(self, comm=None, timeout=2, executor_wait=True):
         start = time()
 
-        @gen.coroutine
-        def _():
+        async def _():
             if self.process is not None:
-                yield self.kill()
-                yield self.instantiate()
+                await self.kill()
+                await self.instantiate()
 
         try:
-            yield gen.with_timeout(timedelta(seconds=timeout), _())
+            await gen.with_timeout(timedelta(seconds=timeout), _())
         except gen.TimeoutError:
             logger.error("Restart timed out, returning before finished")
-            raise gen.Return("timed out")
+            return "timed out"
         else:
-            raise gen.Return("OK")
+            return "OK"
 
     def memory_monitor(self):
         """ Track worker's memory.  Restart if it goes above terminate fraction """
@@ -354,26 +358,31 @@ class Nanny(ServerNode):
             process.terminate()
 
     def is_alive(self):
-        return self.process is not None and self.process.status == "running"
+        return self.process is not None and self.process.is_alive()
 
     def run(self, *args, **kwargs):
         return run(self, *args, **kwargs)
 
-    @gen.coroutine
-    def _on_exit(self, exitcode):
+    def _on_exit_sync(self, exitcode):
+        self.loop.add_callback(self._on_exit, exitcode)
+
+    async def _on_exit(self, exitcode):
         if self.status not in ("closing", "closed"):
             try:
-                yield self.scheduler.unregister(address=self.worker_address)
+                await self.scheduler.unregister(address=self.worker_address)
             except (EnvironmentError, CommClosedError):
                 if not self.reconnect:
-                    yield self.close()
+                    await self.close()
                     return
 
             try:
                 if self.status not in ("closing", "closed", "closing-gracefully"):
                     if self.auto_restart:
                         logger.warning("Restarting worker")
-                        yield self.instantiate()
+                        await self.instantiate()
+                elif self.status == "closing-gracefully":
+                    await self.close()
+
             except Exception:
                 logger.error(
                     "Failed to restart worker after its process exited", exc_info=True
@@ -395,47 +404,39 @@ class Nanny(ServerNode):
         """
         self.status = "closing-gracefully"
 
-    @gen.coroutine
-    def close(self, comm=None, timeout=5, report=None):
+    async def close(self, comm=None, timeout=5, report=None):
         """
         Close the worker process, stop all comms.
         """
-        while self.status == "closing":
-            yield gen.sleep(0.01)
+        if self.status == "closing":
+            await self.finished()
+            assert self.status == "closed"
 
         if self.status == "closed":
-            raise gen.Return("OK")
+            return "OK"
 
         self.status = "closing"
         logger.info("Closing Nanny at %r", self.address)
         self.stop()
         try:
             if self.process is not None:
-                yield self.kill(timeout=timeout)
+                await self.kill(timeout=timeout)
         except Exception:
             pass
         self.process = None
         self.rpc.close()
         self.status = "closed"
         if comm:
-            yield comm.write("OK")
-        yield ServerNode.close(self)
+            await comm.write("OK")
+        await ServerNode.close(self)
 
 
 class WorkerProcess(object):
     def __init__(
-        self,
-        worker_args,
-        worker_kwargs,
-        worker_start_args,
-        silence_logs,
-        on_exit,
-        worker,
-        env,
+        self, worker_kwargs, worker_start_args, silence_logs, on_exit, worker, env
     ):
         self.status = "init"
         self.silence_logs = silence_logs
-        self.worker_args = worker_args
         self.worker_kwargs = worker_kwargs
         self.worker_start_args = worker_start_args
         self.on_exit = on_exit
@@ -447,17 +448,16 @@ class WorkerProcess(object):
         self.worker_dir = None
         self.worker_address = None
 
-    @gen.coroutine
-    def start(self):
+    async def start(self):
         """
         Ensure the worker process is started.
         """
         enable_proctitle_on_children()
         if self.status == "running":
-            raise gen.Return(self.status)
+            return self.status
         if self.status == "starting":
-            yield self.running.wait()
-            raise gen.Return(self.status)
+            await self.running.wait()
+            return self.status
 
         self.init_result_q = init_q = mp_context.Queue()
         self.child_stop_q = mp_context.Queue()
@@ -467,7 +467,6 @@ class WorkerProcess(object):
             target=self._run,
             name="Dask Worker process (from Nanny)",
             kwargs=dict(
-                worker_args=self.worker_args,
                 worker_kwargs=self.worker_kwargs,
                 worker_start_args=self.worker_start_args,
                 silence_logs=self.silence_logs,
@@ -483,10 +482,16 @@ class WorkerProcess(object):
         self.running = Event()
         self.stopped = Event()
         self.status = "starting"
-        yield self.process.start()
-        msg = yield self._wait_until_connected(uid)
+        try:
+            await self.process.start()
+        except OSError:
+            logger.exception("Nanny failed to start process", exc_info=True)
+            self.process.terminate()
+            return
+
+        msg = await self._wait_until_connected(uid)
         if not msg:
-            raise gen.Return(self.status)
+            return self.status
         self.worker_address = msg["address"]
         self.worker_dir = msg["dir"]
         assert self.worker_address
@@ -495,7 +500,7 @@ class WorkerProcess(object):
 
         init_q.close()
 
-        raise gen.Return(self.status)
+        return self.status
 
     def _on_exit(self, proc):
         if proc is not self.process:
@@ -525,7 +530,7 @@ class WorkerProcess(object):
             assert r is not None
             if r != 0:
                 msg = self._death_message(self.process.pid, r)
-                logger.warning(msg)
+                logger.info(msg)
             self.status = "stopped"
             self.stopped.set()
             # Release resources
@@ -541,8 +546,7 @@ class WorkerProcess(object):
             if self.on_exit is not None:
                 self.on_exit(r)
 
-    @gen.coroutine
-    def kill(self, timeout=2, executor_wait=True):
+    async def kill(self, timeout=2, executor_wait=True):
         """
         Ensure the worker process is stopped, waiting at most
         *timeout* seconds before terminating it abruptly.
@@ -553,7 +557,7 @@ class WorkerProcess(object):
         if self.status == "stopped":
             return
         if self.status == "stopping":
-            yield self.stopped.wait()
+            await self.stopped.wait()
             return
         assert self.status in ("starting", "running")
         self.status = "stopping"
@@ -569,19 +573,18 @@ class WorkerProcess(object):
         self.child_stop_q.close()
 
         while process.is_alive() and loop.time() < deadline:
-            yield gen.sleep(0.05)
+            await gen.sleep(0.05)
 
         if process.is_alive():
             logger.warning(
                 "Worker process still alive after %d seconds, killing", timeout
             )
             try:
-                yield process.terminate()
+                await process.terminate()
             except Exception as e:
                 logger.error("Failed to kill worker process: %s", e)
 
-    @gen.coroutine
-    def _wait_until_connected(self, uid):
+    async def _wait_until_connected(self, uid):
         delay = 0.05
         while True:
             if self.status != "starting":
@@ -589,7 +592,7 @@ class WorkerProcess(object):
             try:
                 msg = self.init_result_q.get_nowait()
             except Empty:
-                yield gen.sleep(delay)
+                await gen.sleep(delay)
                 continue
 
             if msg["uid"] != uid:  # ensure that we didn't cross queues
@@ -599,15 +602,14 @@ class WorkerProcess(object):
                 logger.error(
                     "Failed while trying to start worker process: %s", msg["exception"]
                 )
-                yield self.process.join()
+                await self.process.join()
                 raise msg
             else:
-                raise gen.Return(msg)
+                return msg
 
     @classmethod
     def _run(
         cls,
-        worker_args,
         worker_kwargs,
         worker_start_args,
         silence_logs,
@@ -631,12 +633,11 @@ class WorkerProcess(object):
         IOLoop.clear_instance()
         loop = IOLoop()
         loop.make_current()
-        worker = Worker(*worker_args, **worker_kwargs)
+        worker = Worker(**worker_kwargs)
 
-        @gen.coroutine
-        def do_stop(timeout=5, executor_wait=True):
+        async def do_stop(timeout=5, executor_wait=True):
             try:
-                yield worker.close(
+                await worker.close(
                     report=False,
                     nanny=False,
                     executor_wait=executor_wait,
@@ -665,25 +666,32 @@ class WorkerProcess(object):
         t.daemon = True
         t.start()
 
-        @gen.coroutine
-        def run():
+        async def run():
             """
             Try to start worker and inform parent of outcome.
             """
             try:
-                yield worker._start(*worker_start_args)
+                await worker
             except Exception as e:
                 logger.exception("Failed to start worker")
                 init_result_q.put({"uid": uid, "exception": e})
                 init_result_q.close()
             else:
-                assert worker.address
-                init_result_q.put(
-                    {"address": worker.address, "dir": worker.local_dir, "uid": uid}
-                )
-                init_result_q.close()
-                yield worker.wait_until_closed()
-                logger.info("Worker closed")
+                try:
+                    assert worker.address
+                except ValueError:
+                    pass
+                else:
+                    init_result_q.put(
+                        {
+                            "address": worker.address,
+                            "dir": worker.local_directory,
+                            "uid": uid,
+                        }
+                    )
+                    init_result_q.close()
+                    await worker.wait_until_closed()
+                    logger.info("Worker closed")
 
         try:
             loop.run_sync(run)
