@@ -1,5 +1,6 @@
-from distributed.compatibility import MutableMapping
-from distributed.utils import log_errors, tokey
+from collections import MutableMapping
+
+from .utils import log_errors, tokey
 
 
 class PublishExtension(object):
