@@ -1,8 +1,7 @@
-from __future__ import print_function, division, absolute_import
-
 import datetime
 from functools import partial
 import io
+import queue
 import socket
 import sys
 from time import sleep
@@ -14,12 +13,14 @@ from tornado import gen
 from tornado.ioloop import IOLoop
 
 import dask
-from distributed.compatibility import Queue, Empty, PY2
 from distributed.metrics import time
 from distributed.utils import (
     All,
+    Log,
+    Logs,
     sync,
     is_kernel,
+    is_valid_xml,
     ensure_ip,
     str_graph,
     truncate_exception,
@@ -275,8 +276,6 @@ def test_funcname():
 
 def test_ensure_bytes():
     data = [b"1", "1", memoryview(b"1"), bytearray(b"1")]
-    if PY2:
-        data.append(buffer(b"1"))  # noqa: F821
     for d in data:
         result = ensure_bytes(d)
         assert isinstance(result, bytes)
@@ -316,7 +315,7 @@ def assert_running(loop):
     """
     Raise if the given IOLoop is not running.
     """
-    q = Queue()
+    q = queue.Queue()
     loop.add_callback(q.put, 42)
     assert q.get(timeout=1) == 42
 
@@ -325,14 +324,14 @@ def assert_not_running(loop):
     """
     Raise if the given IOLoop is running.
     """
-    q = Queue()
+    q = queue.Queue()
     try:
         loop.add_callback(q.put, 42)
     except RuntimeError:
         # On AsyncIOLoop, can't add_callback() after the loop is closed
         pass
     else:
-        with pytest.raises(Empty):
+        with pytest.raises(queue.Empty):
             q.get(timeout=0.02)
 
 
@@ -548,3 +547,17 @@ def test_warn_on_duration():
 def test_format_bytes_compat():
     # moved to dask, but exported here for compatibility
     from distributed.utils import format_bytes  # noqa
+
+
+def test_logs():
+    d = Logs({"123": Log("Hello"), "456": Log("World!")})
+    text = d._repr_html_()
+    assert is_valid_xml("<div>" + text + "</div>")
+    assert "Hello" in text
+    assert "456" in text
+
+
+def test_is_valid_xml():
+    assert is_valid_xml("<a>foo</a>")
+    with pytest.raises(Exception):
+        assert is_valid_xml("<a>foo")
